library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
entity de2_ps2 is
   port(clkin,rst        :in std_logic;
        ps2_clk,ps2_data :inout std_logic;
        lefbut,rigbut,midbut: buffer std_logic;
		  mouse_irq : buffer std_logic;
		  s,l : out std_logic;
        x_mov,y_mov: out std_logic_vector(9 downto 0));
end entity;
architecture behav of de2_ps2 is
--
signal byte_cnt,delay :std_logic_vector(3 downto 0);
signal lef_latch,rig_latch,mid_latch :std_logic;
signal x_latch,y_latch :std_logic_vector(9 downto 0);
signal ps2_clk_in,ps2_clk_out,ps2_data_in,ps2_data_out :std_logic;
signal shift_reg :std_logic_vector(32 downto 0);
signal clk_div :std_logic_vector(8 downto 0);
signal clk,ce,de :std_logic;
signal dout_reg :std_logic_vector(9 downto 0);
signal cnt :std_logic_vector(7 downto 0);
--enable instruction 0xF4;
constant enable_data :std_logic_vector(9 downto 0):="0111101000";
signal ct :std_logic_vector(5 downto 0);
--
signal ps2_clk_syn0,ps2_clk_syn1,ps2_dat_syn0,ps2_dat_syn1,dout_syn1,dout_syn0 :std_logic;
--type define
type st is (listen,pullclk,pulldata,trans);
signal state :st ;
signal lef_flag : std_logic;

begin

x_mov <= x_latch(9 downto 0);
y_mov <= y_latch(9 downto 0);

process
begin
  wait until clkin='1';
  clk_div<=clk_div+1;
end process;
clk<=clk_div(8);
--sampling frame data;
process(ps2_clk_in,state)
begin
   if state=listen then
      if ps2_clk_in'event and ps2_clk_in='0' then
         shift_reg(31 downto 0)<=shift_reg(32 downto 1);
         shift_reg(32)<=ps2_data_in;
      end if;
   end if;
end process;
--after the controller send a command to device ,it'll sendback a response,with 11 bit.
--so when ct's value  less than 16,no load activity.
process(ps2_clk_in,cnt,state)
begin
   if cnt="11111111" then
      ct<=(others=>'0');
   elsif state=listen then
      if rising_edge(ps2_clk_in) then
         ct<=ct+1;
      end if;
   end if;
end process;
---idle state counter, watchdog like activity;
process(clk)
begin
   if clk'event and clk='1' then
      if (ps2_data_in='1' and ps2_clk_in='1') then
         cnt<=cnt+1;
      else
         cnt<=(others=>'0');
      end if;
   end if;
end process;
----

----latch data from shift_reg;output is of 2's complement;
process(clk)
begin
   if clk'event and clk='1' then
		if mouse_irq = '1' then
			mouse_irq <= '0';
		end if;
      if cnt="00011110" and (ct(5)='1'or ct(4)='1') then
		s <= shift_reg(1);
		l <= lefbut;
      lef_latch<=shift_reg(1);
      rig_latch<=shift_reg(2);
      mid_latch<=shift_reg(3);
		if shift_reg(1) = '1' then
			mouse_irq <= '1';
		end if;
			if ((x_latch >= 617 and shift_reg(5)='0') or (x_latch <= 2 and shift_reg(5)='1')) then 
				x_latch <= x_latch; 
			else 
				x_latch <= x_latch + (shift_reg(5) & shift_reg(5) & shift_reg(18 downto 12) & '0'); 
				if ((x_latch <= 2 or x_latch >= 639) and shift_reg(5)='1') then
					x_latch <= "0000000010";
				elsif ((x_latch >= 623) and shift_reg(5)='0') then
					x_latch <= "1001011111";
				end if;
			end if; 
			if ((y_latch >= 463 and shift_reg(6)='1') or (y_latch <= 2 and shift_reg(6)='0')) then 
				y_latch <= y_latch; 
			else 
				y_latch <= y_latch + (not (shift_reg(6) & shift_reg(6) & shift_reg(29 downto 23) & '0') + "1"); 
				if ((y_latch <= 2 or y_latch >= 479) and shift_reg(6)='0') then
					y_latch <= "0000000010";
				elsif ((y_latch >= 463) and shift_reg(6)='1') then
					y_latch <= "0110111111";
				end if;
			end if;		
		end if;
   end if;
end process;
lefbut<=lef_latch;
rigbut<=rig_latch;
midbut<=mid_latch;
	
--
ps2_clk_syn0<=ps2_clk;
ps2_dat_syn0<=ps2_data;
--tristate output for ps2_clk and ps2_data;
process(ps2_clk,ce,ps2_clk_out)
begin
   if ce='1' then
      ps2_clk<=ps2_clk_out;
   else
      ps2_clk<='Z';
   end if;
end process;
--tristate output of ps2_data;
process(ps2_data,de,ps2_data_out)
begin
   if de='1' then
      ps2_data<=ps2_data_out;
   else
      ps2_data<='Z';
   end if;
end process;
--synchronize
process
begin
   wait until clk='1';
   ps2_clk_syn1<=ps2_clk_syn0;
   ps2_dat_syn1<=ps2_dat_syn0;
   ps2_clk_in<=ps2_clk_syn1;
   ps2_data_in<=ps2_dat_syn1;
end process;
--tranceive 0xF4 to device when you press the rst button,additional debouncing work 
process(clk)
begin
  if clk'event and clk='1' then
   case state is
   when listen =>  if rst='0' and cnt="11111111" then state<=pullclk;
                   end if;
                   ce<='0';de<='0';
   when pullclk => if delay="1100" then state<=pulldata;
                   end if;
                   ce<='1';de<='0';
   when pulldata=> state<=trans;
                   ce<='1';de<='1';
   when trans =>   if byte_cnt="1010" then state<=listen;
                   end if;
                   ce<='0';de<='1';
   when others  => state<=listen;
   end case;
  end if;
end process;
--pull clk low for about 100us
process(clk,state)
begin
   if state=pullclk then
      if clk'event and clk='1' then
         delay<=delay+1;
      end if;
   else 
      delay<=(others=>'0');
   end if;
end process;
--tranceive data;adds up byte
process(ps2_clk_in,state)
begin
  if state=trans then
     if ps2_clk_in'event and ps2_clk_in='0' then
        dout_reg<='0'&dout_reg(9 downto 1);
     end if;
  else
     dout_reg<=enable_data;
  end if;
end process;
process(ps2_clk_in,state)
begin
if state=trans then
     if ps2_clk_in'event and ps2_clk_in='0' then
        byte_cnt<=byte_cnt+1;        
     end if;
  else
     byte_cnt<=(others=>'0');
  end if;
end process;
--output assignment;
ps2_data_out<=dout_reg(0);
ps2_clk_out<='0';                   
--

end behav;