#include <io.h>
#include <system.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/alt_irq.h> // the irq functions
#include <alt_types.h>
#include "start_interface.h"
#include "map1.h"
#include "map2.h"
#include "monster1.h"
#include "boss1.h"
#include "monster2.h"
#include "boss2.h"
#include "tower_glue.h"
#include "tower_green.h"
#include "tower_dart.h"
#include "tower_bomb.h"
#include "bullet.h"
#include "tower_glue_up.h"
#include "tower_green_up.h"
#include "tower_dart_up.h"
#include "tower_bomb_up.h"
#include "bullet_up.h"
#include "glue_effect.h"
#include "health_bar.h"
#include "select.h"
#include "button.h"
#include "tower_upgrade.h"
#include "bullet_effect.h"
#include "number.h"
#include "quit_restart.h"
#include "tower_select.h"
#include "tower_upgrade2.h"
#include "green_sound.h"
#include "glue_sound.h"
#include "dart_sound.h"
#include "bomb_sound.h"
#include "monster_sound.h"
#include "scream_sound.h"

#define IOWR_WRITE_DATA(base, offset, data) \
		IOWR_16DIRECT(base, (offset*2), data)
#define IORD_READ_DATA(base, offset) \
		IORD_16DIRECT(base, (offset*2))
#define IOWR_WRITE_SPEED(base, data) \
		IOWR_16DIRECT(base + 32, 0, data)
#define IOWR_SET_SPRITE(x, y, rom, sprite, phase) \
		IOWR_32DIRECT(SPRITE_CTRL_BASE, 4*((rom<<7) + sprite), (phase<<20) + (y<<10) + (x<<0))
#define IOWR_SET_SOUND(rom, length, sound, en) \
		IOWR_16DIRECT(SOUND_CTRL_BASE, 0, (length<<10) + (rom<<6) + (sound<<1) + (en<<0))
#define SPRITE_IRQ_CLEAR() \
		IOWR_32DIRECT(SPRITE_CTRL_BASE, 0, (1<<31) + 1023 + (1023<<10) )
#define MOUSE_IRQ_DISABLE() \
		IOWR_32DIRECT(MOUSE_CTRL_BASE, 0, 1)
#define MOUSE_IRQ_ENABLE() \
		IOWR_32DIRECT(MOUSE_CTRL_BASE, 0, 0)
#define TIMER_IRQ_CLEAR() \
		IOWR_16DIRECT(TIMER_CTRL_BASE, 0, 0)

/******ram information******/
#define SLOW_MON 	0
#define NORM_MON 	3
#define FAST_MON	6
#define GLUE_TOW 	9
#define GREEN_TOW 	36
#define DART_TOW 	63
#define BOMB_TOW 	66
#define GLUE_BUL	69
#define GREEN_BUL	70
#define DART_BUL	72
#define BOMB_BUL	78
#define GLUE_IMA	159
#define SEL			162
#define FORBID		163
#define MOUSE		164
#define HEALTH		165
#define PAUSE		173
#define SPEED1		175
#define UNMUTE		177
#define UPGRADE 	179
#define MAX	 		180
#define SELL	 	181
#define RANGE	 	182
#define GLUE_EFF1	183
#define GREEN_EFF1	185
#define BOMB_EFF1	187
#define GLUE_EFF2	183
#define GREEN_EFF2	186
#define BOMB_EFF2	188
#define BOSS_SLOW	189
#define BOSS_FAST	192
#define NUMBER_0 	195
#define NUMBER_1 	196
#define NUMBER_2 	197
#define NUMBER_3 	198
#define NUMBER_4 	199
#define NUMBER_5 	200
#define NUMBER_6 	201
#define NUMBER_7 	202
#define NUMBER_8 	203
#define NUMBER_9 	204
#define QUIT 		205
#define UPGRADE2 	208
#define SELL2	 	209
#define SELL3	 	210
#define TOW_SELECT1 	211
#define TOW_SELECT2 	212
#define TOW_SELECT3 	213
#define TOW_SELECT4 	214

/******game information******/
#define GENERATE_TIME1 		15
#define GENERATE_TIME2 		20
#define GENERATE_TIME3 		25
#define GENERATE_TIME4 		30
#define GAME_START			0
#define GAME_MAP_READY		1
#define GAME_PLAY			2
#define GAME_QUIT			3
#define GAME_DONE			4

int generate_time = 30;
int interface_state = GAME_START;
int tow_run = 0;
int tow_en = 0;
int mons_run = 0;
int run_count = 0;
int new_map = 0;
int current_map = 0;

/******path information******/
#define BG1_PATH 	135
#define BG2_PATH 	125

int background_path1[BG1_PATH][2] =
{{0,13},{0,14},
{1,5},{1,7},{1,8},{1,9},{1,13},{1,14},
{2,7},{2,8},{2,9},{2,10},{2,11},{2,12},
{3,2},{3,3},{3,4},{3,7},{3,8},{3,9},{3,10},{3,11},{3,12},
{4,2},{4,3},{4,4},{4,11},{4,12},
{5,2},{5,3},{5,4},{5,9},{5,11},{5,12},
{6,5},{6,6},{6,7},{6,8},{6,9},{6,10},{6,11},{6,12},
{7,5},{7,6},{7,7},{7,8},{7,9},{7,10},{7,11},{7,12},
{8,5},{8,6},{8,9},
{9,5},{9,6},{9,7},{9,8},{9,9},{9,10},{9,11},{9,12},
{10,2},{10,3},{10,4},{10,5},{10,6},{10,7},{10,8},{10,9},{10,10},{10,11},{10,12},
{11,2},{11,3},{11,4},{11,7},{11,11},{11,12},
{12,2},{12,3},{12,4},{12,11},{12,12},
{13,3},{13,4},{13,5},{13,6},{13,7},{13,8},{13,11},{13,12},{13,13},
{14,3},{14,4},{14,5},{14,6},{14,7},{14,8},{14,11},{14,12},
{15,3},{15,4},{15,7},{15,8},{15,11},{15,12},
{16,3},{16,4},{16,9},{16,11},{16,12},
{17,3},{17,4},{17,5},{17,6},{17,7},{17,8},{17,9},{17,10},{17,11},{17,12},
{18,3},{18,4},{18,5},{18,6},{18,7},{18,8},{18,9},{18,10},{18,11},{18,12},{18,13},{18,14},
{19,13},{19,14}
};

int background_path2[BG2_PATH][2] =
{{0,9},{0,10},{0,12},{0,13},{0,14},
{1,2},{1,3},{1,5},{1,6},
{2,2},{2,3},{2,5},{2,6},{2,7},{2,8},{2,9},{2,10},{2,11},{2,12},
{3,5},{3,6},{3,7},{3,8},{3,9},{3,10},{3,11},{3,12},
{3,7},{3,8},{3,9},{3,10},{3,11},{3,12},
{4,2},{4,3},{4,11},{4,12},
{5,2},{5,3},{5,7},{5,8},{5,9},{5,11},{5,12},
{6,7},{6,8},{6,9},{6,11},{6,12},
{7,2},{7,3},{7,7},{7,8},{7,9},{7,11},{7,12},
{8,2},{8,3},{8,4},{8,5},{8,6},{8,7},{8,8},{8,9},{8,10},{8,11},{8,12},
{9,3},{9,4},{9,5},{9,6},{9,7},{9,8},{9,9},{9,10},{9,11},{9,12},
{10,3},{10,4},
{11,3},{11,4},{11,5},{11,6},{11,7},{11,8},{11,9},{11,10},{11,11},{11,12},
{12,3},{12,4},{12,5},{12,6},{12,7},{12,8},{12,9},{12,10},{12,11},{12,12},
{13,11},{13,12},
{14,11},{14,12},
{15,11},{15,12},
{16,3},{16,4},{16,5},{16,6},{16,7},{16,8},{16,9},{16,10},{16,11},{16,12},
{17,3},{17,4},{17,5},{17,6},{17,7},{17,8},{17,9},{17,10},{17,11},{17,12}
};

/******info_bar information******/
int pause_sel = PAUSE;
int speed_sel = SPEED1;
int sound_sel = UNMUTE;
int pause_sel_x = 1023;
int speed_sel_x = 1023;
int sound_sel_x = 1023;
int button_sel_y = 1023;
int pause_en = 0;
int speed_level = 0;
int mute_en = 0;

/******tower information******/
#define GLUE_RANGE 		68
#define GREEN_RANGE 	68
#define DART_RANGE		88
#define BOMB_RANGE		88

#define TOW_SLOW 		80
#define TOW_FAST		40

#define ATTACK_STATE0	0
#define ATTACK_STATE1	1
#define ATTACK_STATE2	2
#define ATTACK_STATE3	3
#define ATTACK_STATE4	4
#define ATTACK_STATE5	5
#define ATTACK_STATE6	6
#define ATTACK_STATE7	7
#define ATTACK_STATE8	8
#define ATTACK_STATE9	9
#define ATTACK_STATE10	10
#define ATTACK_STATE11	11

#define SOUND_GREEN 	0
#define SOUND_GLUE	 	1
#define SOUND_DART		2
#define SOUND_BOMB		3

int tower_num = 0;
int current_tower = 0;
int attack_state = 0;

typedef struct
{
	int x;
	int y;
	int range;
	int cent_x;
	int cent_y;
	int step_x;
	int step_y;
	int speed;
	int image;
	alt_u8 level;
	alt_u8 busy;
	alt_u8 target;
	alt_u8 type;
	alt_u8 phase;
	alt_u8 sound_en;
	alt_u8 sound_type;
	alt_u8 sound_l;
}tower;

tower tow[20];

/******monster information******/
#define SLOW_SPEED1 		3
#define NORM_SPEED1 		5
#define FAST_SPEED1 		7


#define HEALTH_LEVEL1 	2
#define HEALTH_LEVEL2 	4
#define HEALTH_LEVEL3 	8
#define HEALTH_LEVEL4	12
#define HEALTH_LEVEL5	16
#define HEALTH_LEVEL6	20
#define HEALTH_LEVEL7	24
#define HEALTH_LEVEL8	28
#define HEALTH_LEVEL9	32
#define HEALTH_LEVEL10	50

#define SOUND_KILLED	4
#define SOUND_SCREAM	5

int monster_num[20] = {5,5,5,8,8,8,10,10,10,13,13,13,13,10,10,10,8,8,5,5};
int current_monster = 0;

typedef struct
{
	int x;
	int y;
	int cent_x;
	int cent_y;
	int l;
	int image;
	int speed;
	int phase;
	alt_u8 type;
	int health;
	alt_u8 hight;
	alt_u8 kill;
	alt_u8 sound_en;
	alt_u8 sound_type;
	alt_u8 sound_l;
	alt_u8 sound_scream;
	alt_u8 sound_killed;
}monster;

monster mons[20];

/******bullet information******/
typedef struct
{
	int x;
	int y;
	int type;
	int image;
	alt_u8 phase;
}bullet;

bullet bul[20];

/******status information******/
#define GLUE_TIME		90

typedef struct
{
	int x;
	int y;
	alt_u8 time;
	int image;
}status;

status sta[20];

/******health bar information******/
#define HEALTH_TIME		90
typedef struct
{
	int x;
	int y;
	int image;
	alt_u8 en;
	alt_u8 busy;
	alt_u8 time;
	int health;
	alt_u8 damage;
}health;

health heal[20];

/******number information******/
#define WRITE		8
#define GREEN		4
#define YELLOW		0

int life = 9;
int wave = 0;
int money = 500;
int score = 0;
int color_life = YELLOW;
int color_wave = YELLOW;
int color_score = YELLOW;
int color_money = YELLOW;

int life_x[2] = {1023,1023};
int wave_x[2] = {1023,1023};
int money_x[4] = {1023,1023,1023,1023};
int score_x[4] = {1023,1023,1023,1023};
int life_y[2] = {1023,1023};
int wave_y[2] = {1023,1023};
int money_y[4] = {1023,1023,1023,1023};
int score_y[4] = {1023,1023,1023,1023};

int life_disp[2] = {0,0};
int wave_disp[2] = {0,0};
int score_disp[4] = {0,0,0,0};
int money_disp[4] = {0,0,0,0};

/******mouse information******/
#define INIT_STATE		0
#define GAME_STATE		1
#define FORBID_STATE	2
#define BUILD_STATE		3
#define UPGRADE_STATE	4
#define DONE_STATE		5

int mouse = 0;
int mouse_state = 0;
int left_button = 0;
int right_button = 0;
int x_position = 0;
int y_position = 0;
int mouse_sel_x = 1023;
int mouse_sel_y = 1023;
int tow_sel_x = 1023;
int tow_sel_y = 1023;
int upgrade_sel_x = 1023;
int upgrade_sel_y = 1023;
int sell_sel_x = 1023;
int sell_sel_y = 1023;
int range_sel_x1 = 1023;
int range_sel_y1 = 1023;
int range_sel_x2 = 1023;
int range_sel_y2 = 1023;
int range_sel_x3 = 1023;
int range_sel_y3 = 1023;
int range_sel_x4 = 1023;
int range_sel_y4 = 1023;
int mouse_sel = MOUSE;
int upgrade_sel = UPGRADE;
int sell_sel = SELL;
int tower_sel = TOW_SELECT1;

/******wave generation******/
int wave_gen[20][20][3] =
{
{{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL2}},
//WAVE 1
{{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL2}},
//WAVE 2
{{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL2}},
//WAVE 3
{{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL3},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL3},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL3},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL3},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL3},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL3},
{NORM_SPEED1, FAST_MON, HEALTH_LEVEL3},{NORM_SPEED1, FAST_MON, HEALTH_LEVEL3},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL3},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL3},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL3},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL3},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL3}},
//WAVE 4
{{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},
{NORM_SPEED1, FAST_MON, HEALTH_LEVEL8},{NORM_SPEED1, FAST_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8}},
//WAVE 5
{{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL5},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL5},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL5},
{NORM_SPEED1, FAST_MON, HEALTH_LEVEL5},{NORM_SPEED1, FAST_MON, HEALTH_LEVEL5},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL5},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5}},
//WAVE 6
{{FAST_SPEED1, FAST_MON, HEALTH_LEVEL3},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL3},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL3},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL3},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL3},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL3},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL3},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL3},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL3},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL3},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL3},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL3},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL3}},
//WAVE 7
{{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8}},
//WAVE 8
{{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL5},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL5},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL5},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5}},
//WAVE 9
{{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8}},
//WAVE 10
{{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL5},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL5},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL5},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL5},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL5}},
//WAVE 11
{{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL6},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL6},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL6},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL6},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL6},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL6},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL6},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL6},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL6},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL6},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL6},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL6},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL6}},
//WAVE 12
{{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL7},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL7},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL7},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL7},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL7},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL7},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL7},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL7},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL7},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL7},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL7},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL7},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL7}},
//WAVE 13
{{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL5},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8}},
//WAVE 14
{{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL5},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8}},
//WAVE 15
{{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL8},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL10},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL8}},
//WAVE 16
{{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL9},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL9},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL9},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL9},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL9},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL9},
{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL5},{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL5},{FAST_SPEED1, BOSS_FAST, HEALTH_LEVEL5},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL9},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL9},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL9},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL9}},
//WAVE 17
{{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL9},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL9},{SLOW_SPEED1, SLOW_MON, HEALTH_LEVEL9},
{NORM_SPEED1, NORM_MON, HEALTH_LEVEL9},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL9},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL9},
{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL5},{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL5},{FAST_SPEED1, BOSS_FAST, HEALTH_LEVEL5},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL9},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL9},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL9},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL9}},
//WAVE 18
{{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL10},{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL10},{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL10},
{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL10},{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL10},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL10},
{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL10},{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL10},{FAST_SPEED1, BOSS_FAST, HEALTH_LEVEL10},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL10},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL10},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL10},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL10}},
//WAVE 19
{{FAST_SPEED1, BOSS_FAST, HEALTH_LEVEL10},{FAST_SPEED1, BOSS_FAST, HEALTH_LEVEL10},{FAST_SPEED1, BOSS_FAST, HEALTH_LEVEL10},
{FAST_SPEED1, BOSS_FAST, HEALTH_LEVEL10}, {FAST_SPEED1, BOSS_FAST, HEALTH_LEVEL10},{NORM_SPEED1, NORM_MON, HEALTH_LEVEL10},
{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL10},{SLOW_SPEED1, BOSS_SLOW, HEALTH_LEVEL10},{FAST_SPEED1, BOSS_FAST, HEALTH_LEVEL10},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL10},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL10},{FAST_SPEED1, FAST_MON, HEALTH_LEVEL10},
{FAST_SPEED1, FAST_MON, HEALTH_LEVEL10}},
//WAVE 20
};



void sprite_clear(int *x, int *y)
{
	*x = 1023;
	*y = 1023;
}

void game_init()
{
	int i = 0;
	wave = 0;
	life = 9;
	pause_en = 0;
	speed_level = 0;
	mute_en = 0;
	tower_num = 0;
	pause_sel = PAUSE;
	speed_sel = SPEED1;
	sound_sel = UNMUTE;

	sprite_clear(&tow_sel_x, &tow_sel_y);
	sprite_clear(&mouse_sel_x, &mouse_sel_y);
	sprite_clear(&upgrade_sel_x, &upgrade_sel_y);
	sprite_clear(&sell_sel_x, &sell_sel_y);
	sprite_clear(&range_sel_x1, &range_sel_y1);
	sprite_clear(&range_sel_x2, &range_sel_y2);
	sprite_clear(&range_sel_x3, &range_sel_y3);
	sprite_clear(&range_sel_x4, &range_sel_y4);

	sprite_clear(&pause_sel_x, &button_sel_y);
	sprite_clear(&speed_sel_x, &button_sel_y);
	sprite_clear(&sound_sel_x, &button_sel_y);

	sprite_clear(&life_x[0], &life_y[0]);
	sprite_clear(&life_x[1], &life_y[1]);
	sprite_clear(&wave_x[0], &wave_y[0]);
	sprite_clear(&wave_x[1], &wave_y[1]);
	sprite_clear(&score_x[0], &score_y[0]);
	sprite_clear(&score_x[1], &score_y[1]);
	sprite_clear(&score_x[2], &score_y[2]);
	sprite_clear(&score_x[3], &score_y[3]);
	sprite_clear(&money_x[0], &money_y[0]);
	sprite_clear(&money_x[1], &money_y[1]);
	sprite_clear(&money_x[2], &money_y[2]);
	sprite_clear(&money_x[3], &money_y[3]);

	for(i = 0; i < 13; i++)
	{
		sprite_clear(&mons[i].x, &mons[i].y);
		sprite_clear(&tow[i].x, &tow[i].y);
		sprite_clear(&bul[i].x, &bul[i].y);
		sprite_clear(&sta[i].x, &sta[i].y);
		sprite_clear(&heal[i].x, &heal[i].y);
		mons[i].kill = 1;
	}
}

void monster_new(monster *m, health *h, status *s, int speed, int image, int health_level)
{
	m->l = 0;
	m->kill = 0;
	m->speed = speed;
	m->type = image;
	m->image = image;
	if(image <= 100)
		m->hight = 0;
	else
		m->hight = 8;
	m->health = health_level;
	m->sound_type = SOUND_KILLED;
	m->sound_l = 15;
	m->sound_scream = 0;
	m->sound_killed = 0;
	h->health = health_level;
	h->en = 0;
	h->busy = 0;
	h->time = 0;
	s->time = 0;
}

void tower_new(int x, int y, int range, int speed, int tow_image, int bul_image, int sound)
{
	score += 10;
	tow[tower_num].x = x;
	tow[tower_num].y = y;
	tow[tower_num].cent_x = x + 16;
	tow[tower_num].cent_y = y + 16;
	tow[tower_num].range = range;
	tow[tower_num].speed = speed/(speed_level+1);
	tow[tower_num].type = tow_image;
	tow[tower_num].image = tow_image;
	tow[tower_num].level = 1;
	tow[tower_num].busy = 0;
	tow[tower_num].target = 0;
	tow[tower_num].phase = 0;
	tow[tower_num].sound_type = sound;
	tow[tower_num].sound_l = sound;
	tow[tower_num].step_x = 0;
	tow[tower_num].step_y = 0;
	bul[tower_num].type = bul_image;
	bul[tower_num].image = bul_image;
	bul[tower_num].phase = 0;
	tower_num ++;
}

void tower_sell(int current)
{
	score += 5;
	int i;
	if (tow[current].type == GREEN_TOW && tow[current].level == 1)
		money += 90;
	else if (tow[current].type == GREEN_TOW  && tow[current].level == 2)
		money += 200;
	else if (tow[current].type == GLUE_TOW  && tow[current].level == 1)
		money += 90;
	else if (tow[current].type == GLUE_TOW  && tow[current].level == 2)
		money += 200;
	else if (tow[current].type == BOMB_TOW && tow[current].level == 1)
		money += 200;
	else if (tow[current].type == BOMB_TOW && tow[current].level == 2)
		money += 480;
	else if (tow[current].type == DART_TOW && tow[current].level == 1)
		money += 200;
	else if (tow[current].type == DART_TOW && tow[current].level == 2)
		money += 480;

	for(i = current; i < tower_num - 1; i++)
	{
		tow[i].x = tow[i+1].x;
		tow[i].y = tow[i+1].y;
		tow[i].cent_x = tow[i+1].cent_x;
		tow[i].cent_y = tow[i+1].cent_y;
		tow[i].range = tow[i+1].range;
		tow[i].speed = tow[i+1].speed;
		tow[i].type = tow[i+1].type;
		tow[i].image = tow[i+1].image;
		tow[i].level = tow[i+1].level;
		tow[i].busy = tow[i+1].busy;
		tow[i].target = tow[i+1].target;
		tow[i].phase = tow[i+1].phase;
		tow[i].sound_type = tow[i+1].sound_type;
		tow[i].sound_l = tow[i+1].sound_l;
		tow[i].step_x = tow[i+1].step_x;
		tow[i].step_y = tow[i+1].step_y;
		bul[i].type = bul[i+1].type;
		bul[i].image = bul[i+1].image;
		bul[i].phase = bul[i+1].phase;
	}
	sprite_clear(&tow[tower_num - 1].x, &tow[tower_num - 1].y);
	sprite_clear(&bul[tower_num - 1].x, &bul[tower_num - 1].y);
	tower_num --;
}

void monster_update(monster *m, status *s)
{
	if(m -> kill == 0)
	{
		if (m->image == m->type+2)
		{
			m->image = m->type;
			s->image = GLUE_IMA;
		}
		else
		{
			m->image ++;
			s->image ++;
		}
	}
}

void bullet_update(bullet *b)
{
	if(b->image == b->type + 5)
		b->image = b->type;
	else if(b->image < b->type + 5)
		b->image ++;
}

void health_update(health *h, monster *m)
{
	if(h->time != 0)
	{
		h->x = m->x;
		h->y = m->y - 4;
		if(h->time == HEALTH_TIME/(speed_level+1))
		{
			h->health --;

			float health_ratio = (float)h->health/(float)m->health;

			if(health_ratio > 0.875)
				h->image = HEALTH + 1;
			else if(health_ratio > 0.75)
				h->image = HEALTH + 1;
			else if(health_ratio > 0.625)
				h->image = HEALTH + 2;
			else if(health_ratio > 0.5)
				h->image = HEALTH + 3;
			else if(health_ratio > 0.375)
				h->image = HEALTH + 4;
			else if(health_ratio > 0.25)
				h->image = HEALTH + 5;
			else if(health_ratio > 0.125)
				h->image = HEALTH + 6;
			else if(health_ratio > 0.0)
				h->image = HEALTH + 7;

			else if(h->health == 0)
			{
				h->time = 0;
				m->l = 1490;
				m->kill = 1;
				m->sound_killed = 1;
				sprite_clear(&m->x, &m->y);
				sprite_clear(&h->x, &h->y);
			}
		}
		h->time --;
	}
	else
		sprite_clear(&h->x, &h->y);
}

void status_update(status *s, monster *m)
{
	if(s->time != 0)
	{
		if(m->x != 1023 && m->y != 1023)
		{
			s->x = m->x;
			s->y = m->y + m->hight;
		}
		else
			sprite_clear(&s->x, &s->y);
		s->time --;
		if(m->type == SLOW_MON)
			m->speed = SLOW_SPEED1 - 2;
		else if(m->type == NORM_MON)
			m->speed = NORM_SPEED1 - 2;
		else if(m->type == FAST_MON)
			m->speed = FAST_SPEED1 - 3;
		else if(m->type == BOSS_SLOW)
			m->speed = SLOW_SPEED1 - 2;
		else if(m->type == BOSS_FAST)
			m->speed = FAST_SPEED1 - 3;
	}
	else
	{
		sprite_clear(&s->x, &s->y);
		if(m->type == SLOW_MON)
			m->speed = SLOW_SPEED1;
		else if(m->type == NORM_MON)
			m->speed = NORM_SPEED1;
		else if(m->type == FAST_MON)
			m->speed = FAST_SPEED1;
		else if(m->type == BOSS_SLOW)
			m->speed = SLOW_SPEED1;
		else if(m->type == BOSS_FAST)
			m->speed = FAST_SPEED1;
	}
}

void monster_path1_update(monster *m, status *s, health *h)
{

	if(m->kill == 0)

	{
		m->l = m->l + m->speed * (speed_level + 1);

		if(m->l >= 0 && m->l <= 112)
		{
			m->kill = 0;
			m->phase = 0;
			m->cent_x = 96;
			m->cent_y = m->l + 262 - m->hight;
		}
		else if(m->l > 112 && m->l <= 240)
		{
			m->cent_x = m->l - 112 + 96;
			m->cent_y = 374 - m->hight;
		}
		else if(m->l > 240 && m->l <= 432)
		{
			m->cent_x = 224;
			m->cent_y = 432 - m->l + 182 - m->hight;
		}
		else if(m->l > 432 && m->l <= 528)
		{
			m->cent_x = m->l - 432 + 224;
			m->cent_y = 182 - m->hight;
		}
		else if(m->l > 528 && m->l <= 720)
		{
			m->cent_x = 320;
			m->cent_y = m->l -528 + 182 - m->hight;
		}
		else if(m->l > 720 && m->l <= 976)
		{
			m->cent_x = m->l -720 + 320;
			m->cent_y = 374 - m->hight;
		}
		else if(m->l > 976 && m->l <= 1232)
		{
			m->cent_x = 576;
			m->cent_y = 1232 - m->l + 118 - m->hight;
		}
		else if(m->l > 1232 && m->l <= 1360)
		{
			m->cent_x = 1360 - m->l + 448;
			m->cent_y = 118 - m->hight;
			m->phase = 1;
		}
		else if(m->l > 1360 && m->l <= 1475)
		{
			m->cent_x = 448;
			m->cent_y = m->l - 1360 + 118 - m->hight;
		}
		else if(m->l > 1475)
		{
			m->kill = 1;
			if (m->type > 100)
				life = life - 2;
			else
				life --;
			m->sound_scream = 1;
			m->cent_x = 1039;
			m->cent_y = 1039;
		}
		m->x = m->cent_x - 16;
		m->y = m->cent_y - 16;
	}
	else
	{
		m->x = 1023;
		m->y = 1023;
	}
}


void monster_path2_update(monster *m, status *s, health *h)
{

	if(m->kill == 0)

	{
		m->l = m->l + m->speed * (speed_level + 1);

		if(m->l >= 0 && m->l <= 192)
		{
			m->kill = 0;
			m->phase = 0;
			m->cent_x = 96;
			m->cent_y = m->l + 182 - m->hight;
		}
		else if(m->l > 192 && m->l <= 384)
		{
			m->cent_x = m->l - 192 + 96;
			m->cent_y = 374 - m->hight;
		}
		else if(m->l > 384 && m->l <= 640)
		{
			m->cent_x = 288;
			m->cent_y = 640 - m->l + 118 - m->hight;
		}
		else if(m->l > 640 && m->l <= 736)
		{
			m->cent_x = m->l - 640 + 288;
			m->cent_y = 118 - m->hight;
		}
		else if(m->l > 736 && m->l <= 992)
		{
			m->cent_x = 384;
			m->cent_y = m->l - 736 + 118 - m->hight;
		}
		else if(m->l > 992 && m->l <= 1152)
		{
			m->cent_x = m->l - 992 + 384;
			m->cent_y = 374 - m->hight;
		}
		else if(m->l > 1152 && m->l <= 1376)
		{
			m->cent_x = 544;
			m->cent_y = 1376 - m->l + 150 - m->hight;
		}
		else if(m->l > 1376)
		{
			m->kill = 1;
			m->sound_scream = 1;
			if (m->type > 100)
				life = life - 2;
			else
				life --;
			m->cent_x = 1039;
			m->cent_y = 1039;
		}
		m->x = m->cent_x - 16;
		m->y = m->cent_y - 16;
	}
	else
	{
		m->x = 1023;
		m->y = 1023;
	}
}

int main()
{
int i,j,k;
IOWR_WRITE_DATA(MUX_CTRL_BASE, 0, 0x0000);
printf("Welcome to save edward\n");
IOWR_WRITE_SPEED(SRAM_CTRL_BASE, 0x0040);

int offset = 0;

/*******************background***********************/
for(i = offset; i < 153600+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, START_INTERFACE[i-offset]);
offset = i; printf("writing %x\n", offset);

/********************* monster **************************/
for(i = offset; i < 4608+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, MONSTER1[i-offset]);
offset = i; printf("writing %x\n", offset);

/***********************tower glue***********************/
for(i = offset; i < 13824+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, TOWER_GLUE[i-offset]);
offset = i; printf("writing %x\n", offset);

/***********************tower green**********************/
for(i = offset; i < 13824+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, TOWER_GREEN[i-offset]);
offset = i; printf("writing %x\n", offset);

/***********************tower dart ***********************/
for(i = offset; i < 1536+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, TOWER_DART[i-offset]);
offset = i; printf("writing %x\n", offset);

/***********************tower bomb ***********************/
for(i = offset; i < 1536+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, TOWER_BOMB[i-offset]);
offset = i; printf("writing %x\n", offset);

/************************bullet**************************/
for(i = offset; i < 7680+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, BULLET[i-offset]);
offset = i; printf("writing %x\n", offset);

/***********************tower glue up***********************/
for(i = offset; i < 13824+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, TOWER_GLUE_UP[i-offset]);
offset = i; printf("writing %x\n", offset);

/***********************tower green up**********************/
for(i = offset; i < 13824+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, TOWER_GREEN_UP[i-offset]);
offset = i; printf("writing %x\n", offset);

/***********************tower dart up***********************/
for(i = offset; i < 1536+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, TOWER_DART_UP[i-offset]);
offset = i; printf("writing %x\n", offset);

/***********************tower bomb up***********************/
for(i = offset; i < 1536+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, TOWER_BOMB_UP[i-offset]);
offset = i; printf("writing %x\n", offset);

/************************bullet up**************************/
for(i = offset; i < 7680+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, BULLET_UP[i-offset]);
offset = i; printf("writing %x\n", offset);

/*****************glue effect 1,2,3********************/
for(i = offset; i < 1536+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, GLUE_EFFECT[i-offset]);
offset = i; printf("writing %x\n", offset);

/********************select 1,2,3**********************/
for(i = offset; i < 1536+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, SELECT[i-offset]);
offset = i; printf("writing %x\n", offset);

/******************health bar 1-8*********************/
for(i = offset; i < 512+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, HEALTH_BAR[i-offset]);
offset = i; printf("writing %x\n", offset);

/**********************pause*************************/
for(i = offset; i < 3072+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, BUTTON[i-offset]);
offset = i; printf("writing %x\n", offset);

/**********************tower upgrade*************************/
for(i = offset; i < 2048+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, TOWER_UPGRADE[i-offset]);
offset = i; printf("writing %x\n", offset);

/**********************bullet effect*************************/
for(i = offset; i < 3072+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, BULLET_EFFECT[i-offset]);
offset = i; printf("writing %x\n", offset);

/************************* boss ***************************/
for(i = offset; i < 3840+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, BOSS1[i-offset]);
offset = i; printf("writing %x\n", offset);

/*********************** number *************************/
for(i = offset; i < 640+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, NUMBER[i-offset]);
offset = i; printf("writing %x\n", offset);

/********************* quit_restart *********************/
for(i = offset; i < 3072+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, QUIT_RESTART[i-offset]);
offset = i; printf("writing %x\n", offset);

/**********************tower upgrade2*************************/
for(i = offset; i < 1536+offset; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, TOWER_UPGRADE2[i-offset]);
offset = i; printf("writing %x\n", offset);

/******************tower select *********************/
for(i = 257952; i < 2048+257952; i++)
	IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, TOWER_SELECT[i-257952]);
offset = i; printf("writing %x\n", offset);

/******************tower sound *********************/
offset = 0;
for(i = offset; i < 1536 + offset; i++)
	IOWR_WRITE_DATA(RAM_CTRL_BASE, i, GREEN_SOUND[i-offset]);
offset = i;

for(i = offset; i < 1024 + offset; i++)
	IOWR_WRITE_DATA(RAM_CTRL_BASE, i, GLUE_SOUND[i-offset]);
offset = i;

for(i = offset; i < 7168 + offset; i++)
	IOWR_WRITE_DATA(RAM_CTRL_BASE, i, DART_SOUND[i-offset]);
offset = i;

for(i = offset; i < 5120 + offset; i++)
	IOWR_WRITE_DATA(RAM_CTRL_BASE, i, BOMB_SOUND[i-offset]);
offset = i;

for(i = offset; i < 4864 + offset; i++)
	IOWR_WRITE_DATA(RAM_CTRL_BASE, i, MONSTER_SOUND[i-offset]);
offset = i;

for(i = offset; i < 2560 + offset; i++)
	IOWR_WRITE_DATA(RAM_CTRL_BASE, i, SCREAM_SOUND[i-offset]);
offset = i;

IOWR_WRITE_DATA(MUX_CTRL_BASE, 0, 0x000c);

void sprite_interrupt_handler ();
void mouse_interrupt_handler ();
void audio_interrupt_handler ();
void timer_interrupt_handler ();

alt_ic_isr_register(SPRITE_CTRL_IRQ_INTERRUPT_CONTROLLER_ID, SPRITE_CTRL_IRQ, (void*)sprite_interrupt_handler, NULL, NULL);
alt_ic_isr_register(MOUSE_CTRL_IRQ_INTERRUPT_CONTROLLER_ID, MOUSE_CTRL_IRQ, (void*)mouse_interrupt_handler, NULL, NULL);
alt_ic_isr_register(TIMER_CTRL_IRQ_INTERRUPT_CONTROLLER_ID, TIMER_CTRL_IRQ, (void*)timer_interrupt_handler, NULL, NULL);
alt_ic_isr_register(SOUND_CTRL_IRQ_INTERRUPT_CONTROLLER_ID, SOUND_CTRL_IRQ, (void*)audio_interrupt_handler, NULL, NULL);

for(;;)
{

	switch (interface_state)
	{
		case GAME_START:
			game_init();
		break;
		case GAME_MAP_READY:
			game_init();
			color_life = YELLOW;
			color_score = YELLOW;

			IOWR_WRITE_DATA(MUX_CTRL_BASE, 0, 0x0000);
			if(current_map == 1)
			{
				for(i = 0; i < 153600; i++)
					IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, MAP1[i]);

				for(i = 153600; i < 4608+153600; i++)
					IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, MONSTER1[i-153600]);

				for(i = 246784; i < 3840+246784; i++)
					IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, BOSS1[i-246784]);

				money = 500;
				score = 0;
				life_x[0] = 460;
				life_y[0] = 285;
			}
			else if(current_map == 2)
			{
				for(i = 0; i < 153600; i++)
					IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, MAP2[i]);

				for(i = 153600; i < 4608+153600; i++)
					IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, MONSTER2[i-153600]);

				for(i = 246784; i < 3840+246784; i++)
					IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, BOSS2[i-246784]);

				money = 500;
				score = 0;
				life_x[0] = 505;
				life_y[0] = 160;
			}
			IOWR_WRITE_DATA(MUX_CTRL_BASE, 0, 0x000c);

			for(i = 0; i < 13; i++)
			{
				sprite_clear(&tow[i].x, &tow[i].y);
				sprite_clear(&bul[i].x, &bul[i].y);
				sprite_clear(&sta[i].x, &sta[i].y);
				sprite_clear(&heal[i].x, &heal[i].y);
				mons[i].kill = 1;
			}
			wave_x[1] = 340;
			wave_y[1] = 28;
			money_x[1] = 67;
			money_y[1] = 16;
			money_x[2] = 77;
			money_y[2] = 16;
			money_x[3] = 87;
			money_y[3] = 16;
			score_x[3] = 360;
			score_y[3] = 5;
			color_life = GREEN;
			color_score = WRITE;
			interface_state = GAME_PLAY;
			mouse_state = GAME_STATE;
		break;
		case GAME_PLAY:

			//display button
			pause_sel_x = 600;
			speed_sel_x = 510;
			sound_sel_x = 420;
			button_sel_y = 8;

			/***************************generate new wave**************************/
			if(mons_run == 1)
			{
				mons_run = 0;

				//dynamic display the monster status
				for(i = 0; i < monster_num[wave-1]; i++)
					monster_update(&mons[i], &sta[i]);

				if(pause_en == 0)
				{
					if(wave < 20){
						int die = 0;
						for(i = 0; i < monster_num[wave-1]; i++)
						{
							if(mons[i].kill == 1)
								die ++;
						}
						if(die == monster_num[wave-1])
						{
							wave ++;
							generate_time = GENERATE_TIME1/(speed_level + 1);
							monster_new(&mons[0], &heal[0], &sta[0], wave_gen[wave-1][0][0], wave_gen[wave-1][0][1], wave_gen[wave-1][0][2]);
							current_monster = 1;
						}

						if(generate_time == 0)
						{
							if(current_monster < monster_num[wave-1])
								monster_new(&mons[current_monster], &heal[current_monster], &sta[current_monster],
										wave_gen[wave-1][current_monster][0], wave_gen[wave-1][current_monster][1], wave_gen[wave-1][current_monster][2]);
							current_monster ++;
							generate_time = GENERATE_TIME1/(speed_level + 1);
						}
						else
							generate_time --;

						if(current_map == 1)
						{
							for(i = 0; i < monster_num[wave-1]; i++)
								monster_path1_update(&mons[i], &sta[i], &heal[i]);
						}
						else if(current_map == 2)
						{
							for(i = 0; i < monster_num[wave-1]; i++)
								monster_path2_update(&mons[i], &sta[i], &heal[i]);
						}
					}
					else if(wave == 20){
						int die = 0;
						for(i = 0; i < monster_num[wave-1]; i++)
						{
							if(mons[i].kill == 1)
								die ++;
						}
						if(die == monster_num[wave-1])
						{
							interface_state = GAME_QUIT;
						}

						if(generate_time == 0)
						{
							if(current_monster < monster_num[wave-1])
								monster_new(&mons[current_monster], &heal[current_monster], &sta[current_monster],
										wave_gen[wave-1][current_monster][0], wave_gen[wave-1][current_monster][1], wave_gen[wave-1][current_monster][2]);
							current_monster ++;
							generate_time = GENERATE_TIME1/(speed_level + 1);
						}
						else
							generate_time --;

						if(current_map == 1)
						{
							for(i = 0; i < monster_num[wave-1]; i++)
								monster_path1_update(&mons[i], &sta[i], &heal[i]);
						}
						else if(current_map == 2)
						{
							for(i = 0; i < monster_num[wave-1]; i++)
								monster_path2_update(&mons[i], &sta[i], &heal[i]);
						}
					}
				}
			}

			/***************************dissipate the wave**************************/

			if(tow_run == 1)
			{
				tow_run = 0;
				if(tow_en == 1)
					tow_en = 0;
				else
					tow_en ++;

				//dynamic display the bullet status
				for(i = 0; i < tower_num; i++)
				{
					if((tow[i].type == DART_TOW || tow[i].type == BOMB_TOW) && tow_en == 1)
						bullet_update(&bul[i]);
				}

				for(i = 0; i < monster_num[wave-1]; i++)
				{
					status_update(&sta[i], &mons[i]);
					health_update(&heal[i], &mons[i]);

					if(mons[i].sound_scream == 1)
					{
						mons[i].sound_scream = 0;
						mons[i].sound_type = SOUND_SCREAM;
						if(mute_en == 0)
							mons[i].sound_en = 1 - mons[i].sound_en;
						mons[i].sound_l = 15;
					}
					if(mons[i].sound_killed == 1)
					{
						mons[i].sound_killed = 0;
						mons[i].sound_type = SOUND_KILLED;
						if(mute_en == 0)
							mons[i].sound_en = 1 - mons[i].sound_en;
						mons[i].sound_l = 12;
						if (mons[i].type > 100)
						{
							money += 48;
							score += 18;
						}
						else
						{
							money += 24;
							score += 6;
						}
					}
				}

				if(pause_en == 0)
				{
					for(j = 0; j < tower_num; j++)
					{
						int upgrade = (tow[j].level-1)*75;
						int attack_speed = 0;
						int attack_interval = 0;
						int sound_interval = 0;
						int game_speed = speed_level + 1;

						if(tow[j].type != DART_TOW)
							attack_speed = TOW_FAST/tow[j].level;
						else
							attack_speed = TOW_SLOW/tow[j].level;

						attack_interval = 4/(game_speed*tow[j].level);
						sound_interval = tow[j].level + speed_level - 1;

						//find the foremost monster that enter the attack range(when the tower is not busy)
						if(tow[j].busy == 0)
						{
							int foremost = 0;
							int foremost_l = 0;
							int attack_true = 0;

							//clear the bullet if it is not busy
							if(tow[j].type == GLUE_TOW || tow[j].type == GREEN_TOW)
								tow[j].image = tow[j].type + (tow[j].image - tow[j].type - upgrade)%9 + upgrade;
							else
								tow[j].image = tow[j].type + upgrade;

							sprite_clear(&bul[j].x, &bul[j].y);

							for(i = 0; i < monster_num[wave-1]; i++)
							{
								int x_length = abs((mons[i].cent_x-tow[j].cent_x)) * abs((mons[i].cent_x-tow[j].cent_x));
								int y_length = abs((mons[i].cent_y-tow[j].cent_y)) * abs((mons[i].cent_y-tow[j].cent_y));
								if(x_length + y_length <= tow[j].range*tow[j].range)
								{
									if(mons[i].l > foremost_l)
									{
										if((tow[j].type == GLUE_TOW || tow[j].type == GREEN_TOW || tow[j].type == DART_TOW || tow[j].type == BOMB_TOW) && mons[i].kill == 0)
										{
											foremost_l = mons[i].l;
											foremost = i;
											attack_true = 1;
										}
									}
								}
							}
							if(attack_true == 1)
							{
								tow[j].target = foremost;
								if(tow[j].type == DART_TOW)
								{
									float x_diff = mons[foremost].cent_x-tow[j].cent_x;
									float y_diff = mons[foremost].cent_y-tow[j].cent_y;
									tow[j].step_x = (int)((5*x_diff)/sqrt(((x_diff*x_diff)+(y_diff*y_diff))) );
									tow[j].step_y = (int)((5*y_diff)/sqrt(((x_diff*x_diff)+(y_diff*y_diff))) );
								}
								tow[j].busy = 1;
							}
						}

						else
						{
							i = tow[j].target;
							int state = 0;

							//judge the phase of the tower
							if(tow[j].type == GLUE_TOW || tow[j].type == GREEN_TOW)
							{
								if(mons[i].cent_x < tow[j].cent_x  && mons[i].cent_y > tow[j].cent_y)
									tow[j].phase = 0;
								else if(mons[i].cent_x < tow[j].cent_x  && mons[i].cent_y < tow[j].cent_y)
									tow[j].phase = 2;
								else if(mons[i].cent_x > tow[j].cent_x  && mons[i].cent_y < tow[j].cent_y)
									tow[j].phase = 3;
								else if(mons[i].cent_x > tow[j].cent_x  && mons[i].cent_y > tow[j].cent_y)
									tow[j].phase = 1;
							}
							else if(tow[j].type == DART_TOW || tow[j].type == BOMB_TOW)
								tow[j].phase = 0;

							float x_diff = abs(mons[i].cent_x-tow[j].cent_x);
							float y_diff = abs(mons[i].cent_y-tow[j].cent_y);

							if(tow[j].type != DART_TOW)
							{
								if(tow[j].level == 1 && game_speed == 1)
								{
									if(tow[j].speed/attack_interval > 7)
										attack_state = ATTACK_STATE0;
									else if(tow[j].speed/attack_interval > 6)
										attack_state = ATTACK_STATE1;
									else if(tow[j].speed/attack_interval > 5)
										attack_state = ATTACK_STATE4;
									else if(tow[j].speed/attack_interval > 4)
										attack_state = ATTACK_STATE5;
									else if(tow[j].speed/attack_interval > 3)
										attack_state = ATTACK_STATE6;
									else if(tow[j].speed/attack_interval > 2)
										attack_state = ATTACK_STATE7;
									else if(tow[j].speed/attack_interval == 2)
										attack_state = ATTACK_STATE8;
									else if(tow[j].speed/attack_interval > 1)
										attack_state = ATTACK_STATE9;
									else if(tow[j].speed/attack_interval > 0)
										attack_state = ATTACK_STATE10;
									else if(tow[j].speed/attack_interval == 0)
										attack_state = ATTACK_STATE11;
								}
								else if((tow[j].level == 2 && game_speed == 1) || (tow[j].level == 1 && game_speed == 2))
								{
									if(tow[j].speed > 14)
										attack_state = ATTACK_STATE0;
									else if(tow[j].speed > 12)
										attack_state = ATTACK_STATE1;
									else if(tow[j].speed > 10)
										attack_state = ATTACK_STATE4;
									else if(tow[j].speed > 8)
										attack_state = ATTACK_STATE5;
									else if(tow[j].speed > 6)
										attack_state = ATTACK_STATE6;
									else if(tow[j].speed > 4)
										attack_state = ATTACK_STATE7;
									else if(tow[j].speed == 4)
										attack_state = ATTACK_STATE8;
									else if(tow[j].speed > 2)
										attack_state = ATTACK_STATE9;
									else if(tow[j].speed > 0)
										attack_state = ATTACK_STATE10;
									else if(tow[j].speed == 0)
										attack_state = ATTACK_STATE11;
								}
								else if(tow[j].level == 2 && game_speed == 2)
								{
									if(tow[j].speed > 7)
										attack_state = ATTACK_STATE0;
									else if(tow[j].speed > 6)
										attack_state = ATTACK_STATE1;
									else if(tow[j].speed > 5)
										attack_state = ATTACK_STATE4;
									else if(tow[j].speed > 4)
										attack_state = ATTACK_STATE5;
									else if(tow[j].speed > 3)
										attack_state = ATTACK_STATE6;
									else if(tow[j].speed > 2)
										attack_state = ATTACK_STATE7;
									else if(tow[j].speed == 2)
										attack_state = ATTACK_STATE8;
									else if(tow[j].speed > 1)
										attack_state = ATTACK_STATE9;
									else if(tow[j].speed > 0)
										attack_state = ATTACK_STATE10;
									else if(tow[j].speed == 0)
										attack_state = ATTACK_STATE11;
								}

							}
							if(tow[j].type == DART_TOW)
							{
								if(tow[j].level == 1 && game_speed == 1)
								{
									if(tow[j].speed/attack_interval > 19)
										attack_state = ATTACK_STATE0;
									else if(tow[j].speed/attack_interval > 18)
										attack_state = ATTACK_STATE1;
									else if(tow[j].speed/attack_interval == 18)
										attack_state = ATTACK_STATE2;
									else if(tow[j].speed/attack_interval > 0)
										attack_state = ATTACK_STATE3;
									else if(tow[j].speed/attack_interval == 0)
										attack_state = ATTACK_STATE11;
								}
								else if((tow[j].level == 2 && game_speed == 1) || (tow[j].level == 1 && game_speed == 2))
								{
									if(tow[j].speed > 38)
										attack_state = ATTACK_STATE0;
									else if(tow[j].speed > 36)
										attack_state = ATTACK_STATE1;
									else if(tow[j].speed == 36)
										attack_state = ATTACK_STATE2;
									else if(tow[j].speed/2 > 0)
										attack_state = ATTACK_STATE3;
									else if(tow[j].speed/2 == 0)
										attack_state = ATTACK_STATE11;
								}
								else if(tow[j].level == 2 && game_speed == 2)
								{
									if(tow[j].speed > 19)
										attack_state = ATTACK_STATE0;
									else if(tow[j].speed > 18)
										attack_state = ATTACK_STATE1;
									else if(tow[j].speed == 18)
										attack_state = ATTACK_STATE2;
									else if(tow[j].speed > 0)
										attack_state = ATTACK_STATE3;
									else if(tow[j].speed == 0)
										attack_state = ATTACK_STATE11;
								}
							}

							switch (attack_state)
							{
								case ATTACK_STATE0:
									state = 9;
								break;
								case ATTACK_STATE1:
									state = 18;
								break;
								case ATTACK_STATE2:
									state = 0;
									if(mute_en == 0)
										tow[j].sound_en = 1 - tow[j].sound_en;
									tow[j].sound_l = tow[j].sound_type + sound_interval*4;
									bul[j].x = tow[j].x;
									bul[j].y = tow[j].y;
								break;
								case ATTACK_STATE3:
									bul[j].x += tow[j].step_x*(game_speed*tow[j].level);
									bul[j].y += tow[j].step_y*(game_speed*tow[j].level);
									if(bul[j].x <= 8 || bul[j].x >= 632 || bul[j].y <= 48 || bul[j].y >= 472)
										tow[j].speed = 1;
									else
									{
										for(k = 0; k < monster_num[wave-1]; k++)
										{
											if(bul[j].x >= mons[k].x - 32 && bul[j].x <= mons[k].x + 32
											&& bul[j].y <= mons[k].y + 32 && bul[j].y >= mons[k].y - 32)
												heal[k].time = HEALTH_TIME/(speed_level + 1);
										}
									}
								break;
								case ATTACK_STATE4:
									state = 0;
									bul[j].x = (int)(((mons[i].cent_x-tow[j].cent_x)*2/6) + tow[j].cent_x) - 16;
									bul[j].y = (int)(((mons[i].cent_y-tow[j].cent_y)*2/6) + tow[j].cent_y) - 16;
								break;
								case ATTACK_STATE5:
									bul[j].x = (int)(((mons[i].cent_x-tow[j].cent_x)*3/6) + tow[j].cent_x) - 16;
									bul[j].y = (int)(((mons[i].cent_y-tow[j].cent_y)*3/6) + tow[j].cent_y) - 16;
								break;
								case ATTACK_STATE6:
									bul[j].x = (int)(((mons[i].cent_x-tow[j].cent_x)*4/6) + tow[j].cent_x) - 16;
									bul[j].y = (int)(((mons[i].cent_y-tow[j].cent_y)*4/6) + tow[j].cent_y) - 16;
								break;
								case ATTACK_STATE7:
									bul[j].x = (int)(((mons[i].cent_x-tow[j].cent_x)*5/6) + tow[j].cent_x) - 16;
									bul[j].y = (int)(((mons[i].cent_y-tow[j].cent_y)*5/6) + tow[j].cent_y) - 16;
								break;
								case ATTACK_STATE8:
									bul[j].x = mons[tow[j].target].x;
									bul[j].y = mons[tow[j].target].y;
									if(tow[j].type == GLUE_TOW)
										bul[j].image = 	GLUE_EFF1;
									else if(tow[j].type == GREEN_TOW)
										bul[j].image = 	GREEN_EFF1;
									else if(tow[j].type == BOMB_TOW)
										bul[j].image = 	BOMB_EFF1;

									if(mute_en == 0 && tow[j].type != DART_TOW)
										tow[j].sound_en = 1 - tow[j].sound_en;
									tow[j].sound_l = tow[j].sound_type + sound_interval*4;

									if(tow[j].type == GREEN_TOW || tow[j].type == DART_TOW)
										heal[tow[j].target].time = HEALTH_TIME/(speed_level + 1);
									else if(tow[j].type == BOMB_TOW)
									{
										for(k = 0; k < monster_num[wave-1]; k++)
										{
											if(abs(mons[tow[j].target].l - mons[k].l) <= 48)
												heal[k].time = HEALTH_TIME/(speed_level + 1);
										}
									}
								break;
								case ATTACK_STATE9:
									bul[j].x = mons[tow[j].target].x;
									bul[j].y = mons[tow[j].target].y;
									if(tow[j].type == GLUE_TOW)
										bul[j].image = 	GLUE_EFF1;
									else if(tow[j].type == GREEN_TOW)
										bul[j].image = 	GREEN_EFF1;
									else if(tow[j].type == BOMB_TOW)
										bul[j].image = 	BOMB_EFF1;
								break;
								case ATTACK_STATE10:
									if(tow[j].type == GLUE_TOW)
										bul[j].image = 	GLUE_EFF2;
									else if(tow[j].type == GREEN_TOW)
										bul[j].image = 	GREEN_EFF2;
									else if(tow[j].type == BOMB_TOW)
										bul[j].image = 	BOMB_EFF2;
								break;
								case ATTACK_STATE11:
									//if(mute_en == 0 && tow[j].type != DART_TOW)
									//	tow[j].sound_en = 1 - tow[j].sound_en;
									//tow[j].sound_l = tow[j].sound_type + sound_interval*4;
									tow[j].busy = 0;
									bul[j].image = bul[j].type;
									tow[j].speed  = attack_speed/game_speed+1;
									if(tow[j].type == GLUE_TOW)
										sta[tow[j].target].time = GLUE_TIME;
									sprite_clear(&bul[j].x, &bul[j].y);
								break;
							}
							tow[j].speed --;

							float angle1 = x_diff/y_diff;
							float angle2 = y_diff/x_diff;
							if(tow[j].type == GLUE_TOW || tow[j].type == GREEN_TOW)
							{
								if(angle1 >= 0.0 && angle1 <= 0.0875)
									tow[j].image = tow[j].type + state + upgrade;
								else if(angle1 >= 0.0875 && angle1 <= 0.176)
									tow[j].image = tow[j].type + 1 + state + upgrade;
								else if(angle1 >= 0.176 && angle1 <= 0.466)
									tow[j].image = tow[j].type + 2 + state + upgrade;
								else if(angle1 >= 0.466 && angle1 <= 0.70)
									tow[j].image = tow[j].type + 3 + state + upgrade;
								else if(angle1 >= 0.70 && angle1 <= 1)
									tow[j].image = tow[j].type + 4 + state + upgrade;
								else if(angle2 >= 0.70 && angle2 <= 1)
									tow[j].image = tow[j].type + 4 + state + upgrade;
								else if(angle2 >= 0.466 && angle2 <= 0.70)
									tow[j].image = tow[j].type + 5 + state + upgrade;
								else if(angle2 >= 0.176 && angle2 <= 0.466)
									tow[j].image = tow[j].type + 6 + state + upgrade;
								else if(angle2 >= 0.0875 && angle2 <= 0.176)
									tow[j].image = tow[j].type + 7 + state + upgrade;
								else if(angle2 >= 0.0 && angle2 <= 0.0875)
									tow[j].image = tow[j].type + 8 + state + upgrade;
							}
							else if(tow[j].type == DART_TOW || tow[j].type == BOMB_TOW)
								tow[j].image = tow[j].type + state/9 + upgrade;

							if(mons[tow[j].target].kill == 1 && tow[j].type != DART_TOW)
							{
								tow[j].busy = 0;
								bul[j].image = bul[j].type;
								tow[j].speed  = attack_speed/game_speed;
								heal[tow[j].target].en = 0;
								sprite_clear(&bul[j].x, &bul[j].y);
								sprite_clear(&sta[tow[j].target].x, &sta[tow[j].target].y);
							}
						}
					}
				}
			}

		wave_disp[0] = (int)wave/10 + NUMBER_0;
		wave_disp[1] = wave%10 + NUMBER_0;

		life_disp[0] = life + NUMBER_0;

		money_disp[0] = (int)(money/1000) + NUMBER_0;
		money_disp[1] = (int)((money - 1000 * (int)(money/1000))/100) + NUMBER_0;
		money_disp[2] = (int)((money - 100 * (int)(money/100))/10) + NUMBER_0;
		money_disp[3] = money%10 + NUMBER_0;

		score_disp[0] = (int)(score/1000) + NUMBER_0;
		score_disp[1] = (int)((score - 1000 * (int)(score/1000))/100) + NUMBER_0;
		score_disp[2] = (int)((score - 100 * (int)(score/100))/10) + NUMBER_0;
		score_disp[3] = score%10 + NUMBER_0;

		if(wave >= 10)
		{
			wave_x[0] = 330;
			wave_y[0] = 28;
		}
		if(money >= 1000)
		{
			money_x[0] = 57;
			money_y[0] = 16;
			money_x[1] = 67;
			money_y[1] = 16;
			money_x[2] = 77;
			money_y[2] = 16;
			money_x[3] = 87;
			money_y[3] = 16;
		}
		else if(money >= 100)
		{
			money_x[0] = 1023;
			money_y[0] = 1023;
			money_x[1] = 57;
			money_y[1] = 16;
			money_x[2] = 67;
			money_y[2] = 16;
			money_x[3] = 77;
			money_y[3] = 16;
		}
		else if(money >= 10)
		{
			money_x[0] = 1023;
			money_y[0] = 1023;
			money_x[1] = 1023;
			money_y[1] = 1023;
			money_x[2] = 57;
			money_y[2] = 16;
			money_x[3] = 67;
			money_y[3] = 16;
		}
		else
		{
			money_x[0] = 1023;
			money_y[0] = 1023;
			money_x[1] = 1023;
			money_y[1] = 1023;
			money_x[2] = 1023;
			money_y[2] = 1023;
			money_x[3] = 57;
			money_y[3] = 16;
		}
		if(score >= 1000)
		{
			score_x[0] = 330;
			score_y[0] = 5;
			score_x[1] = 340;
			score_y[1] = 5;
			score_x[2] = 350;
			score_y[2] = 5;
			score_x[3] = 360;
			score_y[3] = 5;
		}
		else if(score >= 100)
		{
			score_x[0] = 1023;
			score_y[0] = 1023;
			score_x[1] = 330;
			score_y[1] = 5;
			score_x[2] = 340;
			score_y[2] = 5;
			score_x[3] = 350;
			score_y[3] = 5;
		}
		else if(score >= 10)
		{
			score_x[0] = 1023;
			score_y[0] = 1023;
			score_x[1] = 1023;
			score_y[1] = 1023;
			score_x[2] = 330;
			score_y[2] = 5;
			score_x[3] = 340;
			score_y[3] = 5;
		}
		else
		{
			score_x[0] = 1023;
			score_y[0] = 1023;
			score_x[1] = 1023;
			score_y[1] = 1023;
			score_x[2] = 1023;
			score_y[2] = 1023;
			score_x[3] = 330;
			score_y[3] = 5;
		}
		if(life == 0)
			interface_state = GAME_QUIT;
		break;
		case GAME_QUIT:
			upgrade_sel_x = 272;
			upgrade_sel_y = 208;
			upgrade_sel = QUIT;
			sell_sel_x = 304;
			sell_sel_y = 208;
			sell_sel = QUIT + 1;
			mouse_sel_x = 336;
			mouse_sel_y = 208;
			mouse_sel = QUIT + 2;
			mouse_state = DONE_STATE;
		break;
		case GAME_DONE:
			color_life = YELLOW;
			color_score = YELLOW;
			IOWR_WRITE_DATA(MUX_CTRL_BASE, 0, 0x0000);
			for(i = 0; i < 153600; i++)
				IOWR_WRITE_DATA(SRAM_CTRL_BASE, i, START_INTERFACE[i]);
			IOWR_WRITE_DATA(MUX_CTRL_BASE, 0, 0x000c);
			mouse_state = INIT_STATE;
			interface_state = GAME_START;
		break;
	}
}

printf("Goodbye\n");
return 0;
}

void sprite_interrupt_handler()
{
	SPRITE_IRQ_CLEAR();

	mouse = IORD_32DIRECT(MOUSE_CTRL_BASE, 0);
	x_position = (mouse >> 3) & 0x3ff;
	y_position = (mouse >> 13) & 0x3ff;

	/************************display mouse*****************************/
	IOWR_SET_SPRITE(x_position, y_position, MOUSE, 88, 0);

	/***********************display select*****************************/
	IOWR_SET_SPRITE(mouse_sel_x, mouse_sel_y, mouse_sel, 87, 0);
	IOWR_SET_SPRITE(tow_sel_x, tow_sel_y, tower_sel, 86, 0);

	/************************display button****************************/
	IOWR_SET_SPRITE(pause_sel_x, button_sel_y, pause_sel, 85, 0);
	IOWR_SET_SPRITE(speed_sel_x, button_sel_y, speed_sel, 84, 0);
	IOWR_SET_SPRITE(sound_sel_x, button_sel_y, sound_sel, 83, 0);

	/************************display button****************************/
	IOWR_SET_SPRITE(upgrade_sel_x, upgrade_sel_y, upgrade_sel, 82, 0);
	IOWR_SET_SPRITE(sell_sel_x, sell_sel_y, sell_sel, 81, 0);
	IOWR_SET_SPRITE(range_sel_x1, range_sel_y1, RANGE, 80, 0);
	IOWR_SET_SPRITE(range_sel_x2, range_sel_y2, RANGE, 79, 2);
	IOWR_SET_SPRITE(range_sel_x3, range_sel_y3, RANGE, 78, 3);
	IOWR_SET_SPRITE(range_sel_x4, range_sel_y4, RANGE, 77, 1);

	/***********************display monsters***************************/
	IOWR_SET_SPRITE(mons[0].x, mons[0].y, mons[0].image, 12, mons[0].phase);
	IOWR_SET_SPRITE(mons[1].x, mons[1].y, mons[1].image, 15, mons[1].phase);
	IOWR_SET_SPRITE(mons[2].x, mons[2].y, mons[2].image, 18, mons[2].phase);
	IOWR_SET_SPRITE(mons[3].x, mons[3].y, mons[3].image, 21, mons[3].phase);
	IOWR_SET_SPRITE(mons[4].x, mons[4].y, mons[4].image, 24, mons[4].phase);
	IOWR_SET_SPRITE(mons[5].x, mons[5].y, mons[5].image, 27, mons[5].phase);
	IOWR_SET_SPRITE(mons[6].x, mons[6].y, mons[6].image, 30, mons[6].phase);
	IOWR_SET_SPRITE(mons[7].x, mons[7].y, mons[7].image, 33, mons[7].phase);
	IOWR_SET_SPRITE(mons[8].x, mons[8].y, mons[8].image, 36, mons[8].phase);
	IOWR_SET_SPRITE(mons[9].x, mons[9].y, mons[9].image, 39, mons[9].phase);
	IOWR_SET_SPRITE(mons[10].x, mons[10].y, mons[10].image, 42, mons[10].phase);
	IOWR_SET_SPRITE(mons[11].x, mons[11].y, mons[11].image, 45, mons[11].phase);
	IOWR_SET_SPRITE(mons[12].x, mons[12].y, mons[12].image, 48, mons[12].phase);

	/**********************display glue effects*************************/
	IOWR_SET_SPRITE(sta[0].x, sta[0].y, sta[0].image, 13, 0);
	IOWR_SET_SPRITE(sta[1].x, sta[1].y, sta[1].image, 16, 0);
	IOWR_SET_SPRITE(sta[2].x, sta[2].y, sta[2].image, 19, 0);
	IOWR_SET_SPRITE(sta[3].x, sta[3].y, sta[3].image, 22, 0);
	IOWR_SET_SPRITE(sta[4].x, sta[4].y, sta[4].image, 25, 0);
	IOWR_SET_SPRITE(sta[5].x, sta[5].y, sta[5].image, 28, 0);
	IOWR_SET_SPRITE(sta[6].x, sta[6].y, sta[6].image, 31, 0);
	IOWR_SET_SPRITE(sta[7].x, sta[7].y, sta[7].image, 34, 0);
	IOWR_SET_SPRITE(sta[8].x, sta[8].y, sta[8].image, 37, 0);
	IOWR_SET_SPRITE(sta[9].x, sta[9].y, sta[9].image, 40, 0);
	IOWR_SET_SPRITE(sta[10].x, sta[10].y, sta[10].image, 43, 0);
	IOWR_SET_SPRITE(sta[11].x, sta[11].y, sta[11].image, 46, 0);
	IOWR_SET_SPRITE(sta[12].x, sta[12].y, sta[12].image, 49, 0);

	/*************************display health****************************/
	IOWR_SET_SPRITE(heal[0].x, heal[0].y, heal[0].image, 14, 0);
	IOWR_SET_SPRITE(heal[1].x, heal[1].y, heal[1].image, 17, 0);
	IOWR_SET_SPRITE(heal[2].x, heal[2].y, heal[2].image, 20, 0);
	IOWR_SET_SPRITE(heal[3].x, heal[3].y, heal[3].image, 23, 0);
	IOWR_SET_SPRITE(heal[4].x, heal[4].y, heal[4].image, 26, 0);
	IOWR_SET_SPRITE(heal[5].x, heal[5].y, heal[5].image, 29, 0);
	IOWR_SET_SPRITE(heal[6].x, heal[6].y, heal[6].image, 32, 0);
	IOWR_SET_SPRITE(heal[7].x, heal[7].y, heal[7].image, 35, 0);
	IOWR_SET_SPRITE(heal[8].x, heal[8].y, heal[8].image, 38, 0);
	IOWR_SET_SPRITE(heal[9].x, heal[9].y, heal[9].image, 41, 0);
	IOWR_SET_SPRITE(heal[10].x, heal[10].y, heal[10].image, 44, 0);
	IOWR_SET_SPRITE(heal[11].x, heal[11].y, heal[11].image, 47, 0);
	IOWR_SET_SPRITE(heal[12].x, heal[12].y, heal[12].image, 50, 0);

	/*************************display tower****************************/
	IOWR_SET_SPRITE(tow[0].x, tow[0].y, tow[0].image, 51, tow[0].phase);
	IOWR_SET_SPRITE(tow[1].x, tow[1].y, tow[1].image, 52, tow[1].phase);
	IOWR_SET_SPRITE(tow[2].x, tow[2].y, tow[2].image, 53, tow[2].phase);
	IOWR_SET_SPRITE(tow[3].x, tow[3].y, tow[3].image, 54, tow[3].phase);
	IOWR_SET_SPRITE(tow[4].x, tow[4].y, tow[4].image, 55, tow[4].phase);
	IOWR_SET_SPRITE(tow[5].x, tow[5].y, tow[5].image, 56, tow[5].phase);
	IOWR_SET_SPRITE(tow[6].x, tow[6].y, tow[6].image, 57, tow[6].phase);
	IOWR_SET_SPRITE(tow[7].x, tow[7].y, tow[7].image, 58, tow[7].phase);
	IOWR_SET_SPRITE(tow[8].x, tow[8].y, tow[8].image, 59, tow[8].phase);
	IOWR_SET_SPRITE(tow[9].x, tow[9].y, tow[9].image, 60, tow[9].phase);
	IOWR_SET_SPRITE(tow[10].x, tow[10].y, tow[10].image, 61, tow[10].phase);
	IOWR_SET_SPRITE(tow[11].x, tow[11].y, tow[11].image, 62, tow[11].phase);
	IOWR_SET_SPRITE(tow[12].x, tow[12].y, tow[12].image, 63, tow[12].phase);

	/*************************display bullet****************************/
	IOWR_SET_SPRITE(bul[0].x, bul[0].y, bul[0].image, 64, bul[0].phase);
	IOWR_SET_SPRITE(bul[1].x, bul[1].y, bul[1].image, 65, bul[1].phase);
	IOWR_SET_SPRITE(bul[2].x, bul[2].y, bul[2].image, 66, bul[2].phase);
	IOWR_SET_SPRITE(bul[3].x, bul[3].y, bul[3].image, 67, bul[3].phase);
	IOWR_SET_SPRITE(bul[4].x, bul[4].y, bul[4].image, 68, bul[4].phase);
	IOWR_SET_SPRITE(bul[5].x, bul[5].y, bul[5].image, 69, bul[5].phase);
	IOWR_SET_SPRITE(bul[6].x, bul[6].y, bul[6].image, 70, bul[6].phase);
	IOWR_SET_SPRITE(bul[7].x, bul[7].y, bul[7].image, 71, bul[7].phase);
	IOWR_SET_SPRITE(bul[8].x, bul[8].y, bul[8].image, 72, bul[8].phase);
	IOWR_SET_SPRITE(bul[9].x, bul[9].y, bul[9].image, 73, bul[9].phase);
	IOWR_SET_SPRITE(bul[10].x, bul[10].y, bul[10].image, 74, bul[10].phase);
	IOWR_SET_SPRITE(bul[11].x, bul[11].y, bul[11].image, 75, bul[11].phase);
	IOWR_SET_SPRITE(bul[12].x, bul[12].y, bul[12].image, 76, bul[12].phase);

	/*************************display number****************************/
	IOWR_SET_SPRITE(wave_x[0], wave_y[0], wave_disp[0], 1, color_wave);
	IOWR_SET_SPRITE(wave_x[1], wave_y[1], wave_disp[1], 2, color_wave);

	IOWR_SET_SPRITE(life_x[0], life_y[0], life_disp[0], 3, color_life);

	IOWR_SET_SPRITE(money_x[0], money_y[0], money_disp[0], 4, color_money);
	IOWR_SET_SPRITE(money_x[1], money_y[1], money_disp[1], 5, color_money);
	IOWR_SET_SPRITE(money_x[2], money_y[2], money_disp[2], 6, color_money);
	IOWR_SET_SPRITE(money_x[3], money_y[3], money_disp[3], 7, color_money);

	IOWR_SET_SPRITE(score_x[0], score_y[0], score_disp[0], 8, color_score);
	IOWR_SET_SPRITE(score_x[1], score_y[1], score_disp[1], 9, color_score);
	IOWR_SET_SPRITE(score_x[2], score_y[2], score_disp[2], 10, color_score);
	IOWR_SET_SPRITE(score_x[3], score_y[3], score_disp[3], 11, color_score);

}

void audio_interrupt_handler ()
{
	/*************************play tower sound****************************/
	IOWR_SET_SOUND(tow[0].sound_type, tow[0].sound_l, 0, tow[0].sound_en);
	IOWR_SET_SOUND(tow[1].sound_type, tow[1].sound_l, 1, tow[1].sound_en);
	IOWR_SET_SOUND(tow[2].sound_type, tow[2].sound_l, 2, tow[2].sound_en);
	IOWR_SET_SOUND(tow[3].sound_type, tow[3].sound_l, 3, tow[3].sound_en);
	IOWR_SET_SOUND(tow[4].sound_type, tow[4].sound_l, 4, tow[4].sound_en);
	IOWR_SET_SOUND(tow[5].sound_type, tow[5].sound_l, 5, tow[5].sound_en);
	IOWR_SET_SOUND(tow[6].sound_type, tow[6].sound_l, 6, tow[6].sound_en);
	IOWR_SET_SOUND(tow[7].sound_type, tow[7].sound_l, 7, tow[7].sound_en);
	IOWR_SET_SOUND(tow[8].sound_type, tow[8].sound_l, 8, tow[8].sound_en);
	IOWR_SET_SOUND(tow[9].sound_type, tow[9].sound_l, 9, tow[9].sound_en);
	IOWR_SET_SOUND(tow[10].sound_type, tow[10].sound_l, 10, tow[10].sound_en);
	IOWR_SET_SOUND(tow[11].sound_type, tow[11].sound_l, 11, tow[11].sound_en);
	IOWR_SET_SOUND(tow[12].sound_type, tow[12].sound_l, 12, tow[12].sound_en);

	/*************************play monster sound***************************/
	IOWR_SET_SOUND(mons[0].sound_type, mons[0].sound_l, 13, mons[0].sound_en);
	IOWR_SET_SOUND(mons[0].sound_type, mons[1].sound_l, 14, mons[1].sound_en);
	IOWR_SET_SOUND(mons[0].sound_type, mons[2].sound_l, 15, mons[2].sound_en);
	IOWR_SET_SOUND(mons[0].sound_type, mons[3].sound_l, 16, mons[3].sound_en);
	IOWR_SET_SOUND(mons[4].sound_type, mons[4].sound_l, 17, mons[4].sound_en);
	IOWR_SET_SOUND(mons[5].sound_type, mons[5].sound_l, 18, mons[5].sound_en);
	IOWR_SET_SOUND(mons[6].sound_type, mons[6].sound_l, 19, mons[6].sound_en);
	IOWR_SET_SOUND(mons[7].sound_type, mons[7].sound_l, 20, mons[7].sound_en);
	IOWR_SET_SOUND(mons[8].sound_type, mons[8].sound_l, 21, mons[8].sound_en);
	IOWR_SET_SOUND(mons[9].sound_type, mons[9].sound_l, 22, mons[9].sound_en);
	IOWR_SET_SOUND(mons[10].sound_type, mons[10].sound_l, 23, mons[10].sound_en);
	IOWR_SET_SOUND(mons[11].sound_type, mons[11].sound_l, 24, mons[11].sound_en);
	IOWR_SET_SOUND(mons[12].sound_type, mons[12].sound_l, 25, mons[12].sound_en);
}

void mouse_interrupt_handler()
{
	int i;
	MOUSE_IRQ_DISABLE();

	mouse = IORD_32DIRECT(MOUSE_CTRL_BASE, 0);
	x_position = (mouse >> 3) & 0x3ff;
	y_position = (mouse >> 13) & 0x3ff;
	left_button = mouse & 0x1;
	right_button = (mouse>>1) & 0x1;

	switch (mouse_state)
	{
		case INIT_STATE:
			if(left_button == 1)
			{
				if(x_position >= 22 && x_position < 277 && y_position >= 160 && y_position < 351)
				{
					current_map = 1;
					interface_state = GAME_MAP_READY;
					mouse_state = GAME_STATE;
				}
				else if(x_position >= 361 && x_position < 616 && y_position >= 160 && y_position < 351)
				{
					current_map = 2;
					interface_state = GAME_MAP_READY;
					mouse_state = GAME_STATE;
				}
				else
					mouse_state = INIT_STATE;
			}
		break;
		case GAME_STATE:
			if(left_button == 1)
			{
				if(x_position >= 590 && x_position < 632 && y_position >= 8 && y_position < 40)
				{
					pause_en = 1 - pause_en;
					pause_sel = PAUSE + pause_en;
				}
				else if(x_position >= 560 && x_position < 580 && y_position >= 8 && y_position < 40)
				{
					interface_state = GAME_QUIT;
				}
				else if(x_position >= 470 && x_position < 542 && y_position >= 8 && y_position < 40)
				{
					speed_level = 1 - speed_level;
					generate_time = generate_time/(speed_level + 1);
					speed_sel = SPEED1 + speed_level;
				}
				else if(x_position >= 420 && x_position < 452 && y_position >= 8 && y_position < 40)
				{
					mute_en = 1 - mute_en;
					sound_sel = UNMUTE + mute_en;
				}
				else if(y_position > 64)
				{
					for(i = 0; i < BG1_PATH; i++)
					{
						if(current_map == 1 && x_position >= background_path1[i][0]*32 && x_position < background_path1[i][0]*32 + 32 &&
								y_position >= background_path1[i][1]*32 && y_position < background_path1[i][1]*32 + 32)
						{
							mouse_sel_x = x_position - 6;
							mouse_sel_y = y_position - 6;
							mouse_sel = FORBID;
							mouse_state = FORBID_STATE;
							break;
						}
						if(current_map == 2 && x_position >= background_path2[i][0]*32 && x_position < background_path2[i][0]*32 + 32 &&
								y_position >= background_path2[i][1]*32 && y_position < background_path2[i][1]*32 + 32)
						{
							mouse_sel_x = x_position - 6;
							mouse_sel_y = y_position - 6;
							mouse_sel = FORBID;
							mouse_state = FORBID_STATE;
							break;
						}
					}
					if(i < BG1_PATH)
						break;
					for(i = 0; i < tower_num; i++)
					{
						if(x_position >= tow[i].x && x_position <= tow[i].x+32 && y_position >= tow[i].y && y_position <= tow[i].y + 32)
						{
							if((money >= 150 && (tow[i].type == GLUE_TOW || tow[i].type == GREEN_TOW))
									|| (money >= 300 && (tow[i].type == DART_TOW || tow[i].type == BOMB_TOW)))
							{
								upgrade_sel_x = tow[i].x;
								upgrade_sel_y = tow[i].y -32;
							}
							range_sel_x1 = tow[i].x - (tow[i].range - 16);
							range_sel_y1 = tow[i].y + (tow[i].range - 20);
							range_sel_x2 = tow[i].x - (tow[i].range - 16);
							range_sel_y2 = tow[i].y - (tow[i].range - 20);
							range_sel_x3 = tow[i].x + (tow[i].range - 16);
							range_sel_y3 = tow[i].y - (tow[i].range - 20);
							range_sel_x4 = tow[i].x + (tow[i].range - 16);
							range_sel_y4 = tow[i].y + (tow[i].range - 20);
							if(tow[i].y == 448 && tow[i].x < 608)
							{
								sell_sel_x = tow[i].x + 32;
								sell_sel_y = tow[i].y;
							}
							else if(tow[i].y == 448 && tow[i].x == 608)
							{
								sell_sel_x = tow[i].x - 32;
								sell_sel_y = tow[i].y;
							}
							else
							{
								sell_sel_x = tow[i].x;
								sell_sel_y = tow[i].y +32;
							}
							if(tow[i].level == 2)
								upgrade_sel = MAX;
							else
							{
								if(tow[i].type == GLUE_TOW || tow[i].type == GREEN_TOW)
								{
									upgrade_sel = UPGRADE;
									if(tow[i].level == 1)
										sell_sel = SELL;
									else if(tow[i].level == 2)
										sell_sel = SELL2;
								}
								else
								{
									upgrade_sel = UPGRADE2;
									if(tow[i].level == 1)
										sell_sel = SELL3;
									else if(tow[i].level == 2)
										sell_sel = SELL3;
								}
							}
							current_tower = i;
							mouse_state = UPGRADE_STATE;
							break;
						}
					}
					if(i < tower_num)
						break;
					if(tower_num < 13 && money >= 100)
					{
						mouse_sel = SEL;
						mouse_sel_x = x_position - x_position%32;
						mouse_sel_y = y_position - y_position%32;
						if(money < 120)
						{
							if(x_position >= 608)
								tow_sel_x = mouse_sel_x - 32;
							else
								tow_sel_x = mouse_sel_x + 32;
							tow_sel_y = mouse_sel_y;
							tower_sel = TOW_SELECT1;
						}
						else if(money < 210)
						{
							if(x_position >= 608)
								tow_sel_x = mouse_sel_x - 32;
							else
								tow_sel_x = mouse_sel_x + 32;
							tow_sel_y = mouse_sel_y - 32;
							tower_sel = TOW_SELECT2;
						}
						else if(money < 240)
						{
							if(x_position >= 608)
								tow_sel_x = mouse_sel_x - 32;
							else
								tow_sel_x = mouse_sel_x + 32;
							if(y_position >= 448)
								tow_sel_y = mouse_sel_y - 64;
							else
								tow_sel_y = mouse_sel_y - 32;
							tow_sel_y = mouse_sel_y - 32;
							tower_sel = TOW_SELECT3;
						}
						else
						{
							if(x_position >= 608)
								tow_sel_x = mouse_sel_x - 32;
							else
								tow_sel_x = mouse_sel_x + 32;
							if(y_position <= 96)
								tow_sel_y = mouse_sel_y - 32;
							else if(y_position >= 448)
								tow_sel_y = mouse_sel_y - 96;
							else
								tow_sel_y = mouse_sel_y - 64;
							tower_sel = TOW_SELECT4;
						}
						mouse_state = BUILD_STATE;
					}
					else
						mouse_state = GAME_STATE;
				}
			}
		break;
		case FORBID_STATE:
			sprite_clear(&mouse_sel_x, &mouse_sel_y);
			mouse_state = GAME_STATE;
		break;
		case BUILD_STATE:
			if(left_button == 1)
			{
				if(x_position >= tow_sel_x && x_position <= tow_sel_x + 32 && y_position >= tow_sel_y && y_position <= tow_sel_y + 32)
				{
					tower_new(mouse_sel_x, mouse_sel_y, GREEN_RANGE, TOW_FAST,GREEN_TOW, GREEN_BUL, SOUND_GREEN);
					money -= 100;
				}
				else if(x_position >= tow_sel_x && x_position <= tow_sel_x + 32 && y_position >= tow_sel_y + 32 && y_position <= tow_sel_y + 64 && tower_sel >=212)
				{
					tower_new(mouse_sel_x, mouse_sel_y, GLUE_RANGE, TOW_FAST, GLUE_TOW, GLUE_BUL, SOUND_GLUE);
					money -= 120;
				}
				else if(x_position >= tow_sel_x && x_position <= tow_sel_x + 32 && y_position >= tow_sel_y + 64 && y_position <= tow_sel_y + 96 && tower_sel >=213)
				{
					tower_new(mouse_sel_x, mouse_sel_y, BOMB_RANGE, TOW_FAST, BOMB_TOW, BOMB_BUL, SOUND_BOMB);
					money -= 210;
				}
				else if(x_position >= tow_sel_x && x_position <= tow_sel_x + 32 && y_position >= tow_sel_y + 96 && y_position <= tow_sel_y + 128 && tower_sel ==214)
				{
					tower_new(mouse_sel_x, mouse_sel_y, DART_RANGE, TOW_SLOW, DART_TOW, DART_BUL, SOUND_DART);
					money -= 240;
				}
				sprite_clear(&tow_sel_x, &tow_sel_y);
				sprite_clear(&mouse_sel_x, &mouse_sel_y);
				mouse_state = GAME_STATE;
			}
		break;
		case UPGRADE_STATE:
			if(left_button == 1)
			{
				if(x_position >= upgrade_sel_x && x_position <= upgrade_sel_x + 32 && y_position >= upgrade_sel_y && y_position <= upgrade_sel_y + 32
						&& tow[current_tower].level == 1)
				{
					if(tow[current_tower].type == GREEN_TOW || tow[current_tower].type == GLUE_TOW)
						money -= 150;
					else
						money -= 300;
					tow[current_tower].level = 2;
					tow[current_tower].image += 75;
					tow[current_tower].range += (tow[current_tower].level-1)*20;
					bul[current_tower].type += 75;
					bul[current_tower].image += 75;
				}
				else if(x_position >= sell_sel_x && x_position <= sell_sel_x + 32 && y_position >= sell_sel_y && y_position <= sell_sel_y + 32)
					tower_sell(current_tower);
				sprite_clear(&sell_sel_x, &sell_sel_y);
				sprite_clear(&upgrade_sel_x, &upgrade_sel_y);
				sprite_clear(&range_sel_x1, &range_sel_y1);
				sprite_clear(&range_sel_x2, &range_sel_y2);
				sprite_clear(&range_sel_x3, &range_sel_y3);
				sprite_clear(&range_sel_x4, &range_sel_y4);
				mouse_state = GAME_STATE;
			}
		break;
		case DONE_STATE:
			if(left_button == 1)
			{
				if(x_position >= 272 && x_position <= 368 && y_position >= 208 && y_position <= 240)
				{
					sprite_clear(&mouse_sel_x, &mouse_sel_y);
					sprite_clear(&sell_sel_x, &sell_sel_y);
					sprite_clear(&upgrade_sel_x, &upgrade_sel_y);
					interface_state = GAME_MAP_READY;
					mouse_state = GAME_STATE;
				}
				else if(x_position >= 288 && x_position <= 352 && y_position >= 240 && y_position <= 272)
				{
					sprite_clear(&mouse_sel_x, &mouse_sel_y);
					sprite_clear(&sell_sel_x, &sell_sel_y);
					sprite_clear(&upgrade_sel_x, &upgrade_sel_y);
					interface_state = GAME_DONE;
					mouse_state = INIT_STATE;
				}
			}
		break;
	}
	MOUSE_IRQ_ENABLE();
}

void timer_interrupt_handler ()
{
	TIMER_IRQ_CLEAR();
	tow_run = 1;
	if(run_count == 3)
	{
		run_count = 0;
		mons_run = 1;
	}
	else
		run_count ++;
}
